from neuralspace.transcription.constants import DOMAIN, SAMPLE_RATE, SUBURL
from neuralspace.transcription.language_map import LANGUAGE_MAP


def get_sample_rate_and_suburl_from_language(language: str, domain: str):
    sub_url = None
    sample_rate = None
    model_map = LANGUAGE_MAP
    language_models = model_map.get(language, None)
    for model in language_models:
        if model[DOMAIN] == domain:
            sub_url = model.get(SUBURL, None)
            sample_rate = model.get(SAMPLE_RATE, None)

    return sub_url, sample_rate



import asyncio
from typing import Text

import click

from neuralspace.apis import get_async_http_session
from neuralspace.ner.constants import SUPPORTED_LANGUAGES
from neuralspace.transcription.apis import get_languages, list_all_devices, start_stream
from neuralspace.transcription.utils import get_sample_rate_and_suburl_from_language
from neuralspace.utils import setup_logger


@click.group(name="transcription")
def transcription():
    pass


@transcription.command(
    name="stream",
)
@click.option(
    "-L",
    "--language",
    type=click.STRING,
    required=True,
    help="language of the audio file",
)
@click.option(
    "-dom",
    "--domain",
    type=click.STRING,
    required=False,
    default="general",
    help="Set to the spefic domain you wish to choose."
    " If not set, a general purpose model will be used",
)
@click.option(
    "-d",
    "--device",
    type=click.INT,
    required=True,
    help="device number of microphone",
)
@click.option(
    "-l", "--log-level", type=click.Choice(["INFO", "DEBUG", "ERROR"]), default="INFO"
)
def stream(language: Text, domain: str, device: int, log_level: Text):
    setup_logger(log_level=log_level)
    if not language:
        raise ValueError(
            f"Language is mandatory. You can select from {SUPPORTED_LANGUAGES}"
        )
    if not device:
        raise ValueError(
            "Run neuralspace transription list-devices and see device number of your microphone."
            "Set -d as the device number."
        )
    sub_url, sample_rate = get_sample_rate_and_suburl_from_language(language, domain)
    if not sub_url or not sample_rate:
        raise ValueError("Choose model language and domain carefully.")
    loop = asyncio.new_event_loop()
    loop.run_until_complete(start_stream(sub_url, sample_rate, device))
    loop.run_until_complete(get_async_http_session().close())


@transcription.command(name="list-devices")
def list_devices():
    list_all_devices()


@transcription.command(name="get-languages")
@click.option(
    "-l", "--log-level", type=click.Choice(["INFO", "DEBUG", "ERROR"]), default="INFO"
)
def translation_get_languages(log_level: Text):
    setup_logger(log_level=log_level)
    loop = asyncio.new_event_loop()
    loop.run_until_complete(get_languages())
    loop.run_until_complete(get_async_http_session().close())



from typing import Text

from ruamel.yaml.constructor import DuplicateKeyError

from neuralspace.nlu.converters.rasa.exceptions import YamlSyntaxException
from neuralspace.nlu.converters.rasa.shared.constants import (
    PACKAGE_NAME,
    RESPONSES_SCHEMA_FILE,
    SCHEMA_EXTENSIONS_FILE,
)
from neuralspace.nlu.converters.rasa.shared.io import (
    YamlValidationException,
    read_yaml,
    read_yaml_file,
)


def validate_yaml_schema(yaml_file_content: Text, schema_path: Text) -> None:
    """
    Validate yaml content.

    Args:
        yaml_file_content: the content of the yaml file to be validated
        schema_path: the schema of the yaml file
    """
    from pykwalify.core import Core
    from pykwalify.errors import SchemaError
    from ruamel.yaml import YAMLError
    import pkg_resources
    import logging

    log = logging.getLogger("pykwalify")
    log.setLevel(logging.CRITICAL)

    try:
        # we need "rt" since
        # it will add meta information to the parsed output. this meta information
        # will include e.g. at which line an object was parsed. this is very
        # helpful when we validate files later on and want to point the user to the
        # right line
        source_data = read_yaml(yaml_file_content, reader_type=["safe", "rt"])
    except (YAMLError, DuplicateKeyError) as e:
        raise YamlSyntaxException(underlying_yaml_exception=e)

    schema_file = pkg_resources.resource_filename(PACKAGE_NAME, schema_path)
    schema_utils_file = pkg_resources.resource_filename(
        PACKAGE_NAME, RESPONSES_SCHEMA_FILE
    )
    schema_extensions = pkg_resources.resource_filename(
        PACKAGE_NAME, SCHEMA_EXTENSIONS_FILE
    )

    # Load schema content using our YAML loader as `pykwalify` uses a global instance
    # which can fail when used concurrently
    schema_content = read_yaml_file(schema_file)
    schema_utils_content = read_yaml_file(schema_utils_file)
    schema_content = dict(schema_content, **schema_utils_content)

    c = Core(
        source_data=source_data,
        schema_data=schema_content,
        extensions=[schema_extensions],
    )

    try:
        c.validate(raise_exception=True)
    except SchemaError:
        raise YamlValidationException(
            "Please make sure the file is correct and all "
            "mandatory parameters are specified. Here are the errors "
            "found during validation",
            c.errors,
            content=source_data,
        )



from multiprocessing import Queue
from threading import Thread
from typing import Text

import sounddevice as sd
import websocket

from neuralspace.transcription.constants import TRANSCRIPTION_STREAMING_URL


class WebsManager:
    def __init__(self, webs_url: Text) -> None:
        self.webs_url = webs_url

        self.ws = websocket.WebSocket()

    def __enter__(self):
        self.ws.connect(
            self.webs_url,
            timeout=600,
        )
        return self.ws

    def __exit__(self, *exc):
        self.ws.close()
        self.ws.shutdown()


FLAG = True


def start_websocket_stream(  # noqa: C901
    host: str,
    client_id: int,
    chunksize: int,
    ingress: str,
    device: int,
    sample_rate: int,
    auth_token: str,
    output_queue: Queue = None,
):
    q = Queue()

    def callback(data, frames, time, status):
        """This is called (from a separate thread) for each audio block."""
        q.put(bytes(data))

    formed_url = (
        f"wss://{host}/{ingress}/{TRANSCRIPTION_STREAMING_URL}/{auth_token}/{client_id}"
    )
    global FLAG

    def listen(ws, queue: Queue):
        try:
            while FLAG:
                transcription = ws.recv()

                transcription = transcription.encode("raw_unicode_escape").decode(
                    "unicode_escape"
                )

                print("Transcription :: ", transcription)
                if queue:
                    queue.put(transcription)
        except Exception:
            print("Closing listening thread.")

    with WebsManager(formed_url) as ws:
        t = Thread(target=listen, args=[ws, output_queue], daemon=True)
        t.start()
        try:
            with sd.RawInputStream(
                samplerate=sample_rate,
                blocksize=chunksize,
                device=device,
                dtype="int16",
                channels=1,
                callback=callback,
            ):

                while True:
                    data = q.get()
                    if not data:
                        FLAG = False
                        break

                    ws.send_binary(data)

        except KeyboardInterrupt:
            print("\nDone")



from neuralspace.transcription.apis import start_transcription
from neuralspace.transcription.utils import get_sample_rate_and_suburl_from_language


class NSStreamer:
    def __init__(self, output_queue) -> None:

        self.output_queue = output_queue

    def start_stream(self, language: str, specialization: str, device: int):
        sub_url, sample_rate = get_sample_rate_and_suburl_from_language(
            language, specialization
        )
        if not sub_url or not sample_rate:
            raise ValueError("Choose model language and specialization carefully.")
        start_transcription(sub_url, sample_rate, device, self.output_queue)



from multiprocessing import Queue

from neuralspace.transcription.stream import NSStreamer

q = Queue()
streamer = NSStreamer(q)
streamer.start_stream("en", "medical", 23)
while True:
    try:
        print("output :: ", q.get())
    except KeyboardInterrupt:
        exit()



LANGUAGE_MAP = {
    "en": [
        {
            "modelPath": "stt_model/en/vosk/11-05-2022-06-38-vosk-big-model-light-conf-en-us-0.22.zip",
            "domain": "general",
            "accent": "american",
            "suburl": "en-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.\
                 This is the recommended default choice.",
        },
        {
            "modelPath": "stt_model/en/vosk_hub/19-05-2022-03-24-vosk-en-medical-mixed-0.9-us-0.22.zip",
            "accent": "american",
            "domain": "medical",
            "suburl": "en-medical-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need accurate transcriptions especially for medical terms \
                along with commonly spoken words.",
        },
        {
            "modelPath": "stt_model/en/vosk_hub/13-05-2022-07-11-vosk-en-finance-mixed-0.7-us-0.22.zip",
            "accent": "american",
            "domain": "finance",
            "suburl": "en-finance-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need accurate transcriptions especially for financial \
                terms along with commonly spoken words.",
        },
        {
            "modelPath": "stt_model/en-in/vosk_hub/19-05-2022-14-18-vosk-model-en-in-0.5.zip",
            "suburl": "en-in-16",
            "domain": "indian",
            "sampleRate": 16000,
            "description": "Select this domain if you need accurate transcriptions for Indian accented speech.",
        },
    ],
    "hi": [
        {
            "modelPath": "stt_model/hi/vosk_hub/13-05-2022-06-09-vosk-model-hi-0.22.zip",
            "domain": "general",
            "suburl": "hi-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        },
    ],
    "sv": [
        {
            "modelPath": "stt_model/sv/vosk_hub/13-05-2022-07-20-vosk-model-small-sv-rhasspy-0.15.zip",
            "domain": "general",
            "suburl": "sv-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "ar": [
        {
            "modelPath": "stt_model/ar/vosk_hub/13-05-2022-05-08-vosk-model-ar-mgb2-0.4.zip",
            "domain": "general",
            "suburl": "ar-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "ru": [
        {
            "modelPath": "stt_model/ru/vosk_hub/13-05-2022-07-02-vosk-model-ru-0.22.zip",
            "domain": "general",
            "suburl": "ru-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "fr": [
        {
            "modelPath": "stt_model/fr/vosk_hub/13-05-2022-06-38-vosk-model-fr-0.22.zip",
            "domain": "general",
            "suburl": "fr-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "tl": [
        {
            "modelPath": "stt_model/ph/vosk_hub/13-05-2022-07-08-vosk-model-tl-ph-generic-0.6.zip",
            "domain": "general",
            "suburl": "ph-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "uk": [
        {
            "modelPath": "stt_model/uk/vosk_hub/13-05-2022-07-03-vosk-model-uk-v3.zip",
            "domain": "general",
            "suburl": "uk-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "de": [
        {
            "modelPath": "stt_model/de/vosk_hub/13-05-2022-07-01-vosk-model-de-0.21.zip",
            "domain": "general",
            "suburl": "de-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "el": [
        {
            "modelPath": "stt_model/el/vosk_hub/13-05-2022-06-35-vosk-model-el-gr-0.7.zip",
            "domain": "general",
            "suburl": "el-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "fa": [
        {
            "modelPath": "stt_model/fa/vosk_hub/13-05-2022-06-36-vosk-model-fa-0.5.zip",
            "domain": "general",
            "suburl": "fa-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "nl": [
        {
            "modelPath": "stt_model/nl/vosk_hub/19-05-2022-15-16-vosk-model-nl-spraakherkenning-0.6.zip",
            "domain": "general",
            "suburl": "nl-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "pt": [
        {
            "modelPath": "stt_model/pt/vosk_hub/20-04-2022-14-18-vosk-model-small-pt-0.3.zip",
            "domain": "general",
            "suburl": "pt-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "es": [
        {
            "modelPath": "stt_model/es/vosk_hub/20-04-2022-14-24-vosk-model-small-es-0.22.zip",
            "domain": "general",
            "suburl": "es-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "ca": [
        {
            "modelPath": "stt_model/ca/vosk_hub/20-04-2022-14-28-vosk-model-small-ca-0.4.zip",
            "domain": "general",
            "suburl": "ca-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "cs": [
        {
            "modelPath": "stt_model/cs/vosk_hub/20-04-2022-14-55-vosk-model-small-cs-0.4-rhasspy.zip",
            "domain": "general",
            "suburl": "cs-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "eo": [
        {
            "modelPath": "stt_model/eo/vosk_hub/19-05-2022-15-04-vosk-model-small-eo-0.22.zip",
            "domain": "general",
            "suburl": "eo-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "ja": [
        {
            "modelPath": "stt_model/ja/vosk_hub/19-05-2022-15-05-vosk-model-small-ja-0.22.zip",
            "domain": "general",
            "suburl": "ja-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "kk": [
        {
            "modelPath": "stt_model/kz/vosk_hub/13-05-2022-06-45-vosk-model-kz-0.15.zip",
            "domain": "general",
            "suburl": "kz-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "tr": [
        {
            "modelPath": "stt_model/tr/vosk_hub/19-05-2022-14-53-vosk-model-small-tr-0.3.zip",
            "domain": "general",
            "suburl": "tr-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "it": [
        {
            "modelPath": "stt_model/it/vosk_hub/21-04-2022-08-19-vosk-model-small-it-0.4.zip",
            "domain": "general",
            "suburl": "it-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "vn": [
        {
            "modelPath": "stt_model/vn/vosk_hub/19-05-2022-14-57-vosk-model-small-vn-0.3.zip",
            "domain": "general",
            "suburl": "vn-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "zh": [
        {
            "modelPath": "stt_model/cn/vosk_hub/19-05-2022-14-16-vosk-model-cn-kaldi-multicn-0.15.zip",
            "domain": "general",
            "suburl": "cn-16",
            "sampleRate": 16000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
    "or": [
        {
            "modelPath": "stt_model/or/vosk_hub/30-05-2022-07-19-in-or-mucs-char-lmor1-musan-8k.zip",
            "domain": "general",
            "suburl": "or-8",
            "sampleRate": 8000,
            "description": "Select this domain if you need transcriptions that are accurate across domains.",
        }
    ],
}
